import java.util.*;
import java.awt.Graphics;
import java.awt.Graphics2D;

/**
 * Clase que almacena y actualiza los componentes existentes
 */
public class BlockSpringConfiguration  {
   private LabPanel labPanel;
   private Vector <PhysicsElement> phyElements; // Vector is a Container
         // it is like an array, but we can't use [i] for accessing.
         // it can grow as we need it.
   /**
    * Inicializacion del objeto, crea un arreglo de objetos PhysicsElement.
    */
   public BlockSpringConfiguration (){
      phyElements = new Vector <PhysicsElement>();
   }
   /**
    * Metodo para especificar el panel del frame en el que se trabaja.
    * @param lp panel de trabajo.
    */
   public void setLabPanel( LabPanel lp){
      labPanel = lp;
   }
   /**
    * Metodo de calculo de siguiente estado de la simulacion para cada elemento del arreglo.
    * @param delta_t constante de tiempo de la simulacion.
    * @param gravity aceleracion de gravedad.
    */
   public void computeNextState(float delta_t, float gravity) {
      for (int i=0; i<phyElements.size(); i++)
         phyElements.get(i).computeNextState(delta_t,gravity);
   }
   /**
    * Metodo de calculo del proximo estado de la simulacion para cada elemento del arreglo.
    * @param delta_t constante de tiempo de la simulacion.
    */
   public void checkNext(float delta_t) {
      for (int i=0; i<phyElements.size(); i++)
         phyElements.get(i).checkNext(delta_t);
   }
   /**
    * actualizacion del estado de los elementos.
    * @param damp presencia o no de amortiguacion.
    */
   public void updateState(boolean damp) {
      for (int i=0; i<phyElements.size(); i++)
         phyElements.get(i).updateState(damp);
      if (labPanel != null)
         labPanel.repaint();
   }
   /**
    * Metodo para agregar elementos en el panel grafico y visualizarlos.
    * @param e elemento a agregar.
    */
   public void add(PhysicsElement e) {
      phyElements.add(e);
      if (labPanel != null)
         labPanel.repaint();
   }
   /**
    * Metodo para remover elementos en el panel grafico y removerlos de la vista del panel.
    * @param e elemento a remover.
    */
   public void remove(PhysicsElement e) {
      phyElements.remove(e);
      if (labPanel != null)
         labPanel.repaint();
   }
   /**
    * Metodo para invocar el repintado de cada elemento del arreglo.
    * @param g2d elemento grafico del objeto 2D.
    */
   public void paintComponent(Graphics2D g2d){
      for (int i=0; i<phyElements.size(); i++)
         if (phyElements.get(i) instanceof Drawable)
            ((Drawable) phyElements.get(i)).draw(g2d);
   }
   /**
    * Metodo para encontrar el elemento a mover.
    * @param x coordenada x donde buscar.
    * @param y coordenada y donde buscar.
    */
   public Draggable find(int x, int y) {
      for (int i=0; i<phyElements.size(); i++)
         if (phyElements.get(i) instanceof Draggable) {
            Draggable element =(Draggable) (phyElements.get(i));
            if (element.contains(x,y)) return element;
         }
      return null;
   }
   /**
    * Metodo para encontrar el elemento bloque.
    * @param x coordenada x donde buscar.
    * @param y coordenada y donde buscar.
    */
   public GBlock findBlock(double x, double y) {
      for (int i=0; i<phyElements.size(); i++)
         if (phyElements.get(i) instanceof GBlock) {
            GBlock block =(GBlock) (phyElements.get(i));
            if (block.contains(x,y)) return block;
         }
      return null;
   }  
}