import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
/**
 * Clase que realiza la simulacion y solicita la actualizacion de los valores
 */
public class Simulator implements ActionListener {
   private BlockSpringConfiguration bsConfig; // arreglo de objetos graficos de los elementos fisicoselementos graficos.
   private float t;  // Simulation time time
   private Timer pase; // paso de simulacion.
   private float gravity; //meters/seconds^2
   private float delta_t;         // in seconds
   private float samplingPeriod;  // in seconds
   private boolean damp; // amortiguacion

    /**
    * Constructor de la simulacion, recibe configuracion de elementos y se incializa con valores por defecto.
    * @param bsc Configuracion de elementos a simular.
    */
   public Simulator (BlockSpringConfiguration bsc) {
      t=0;
      bsConfig = bsc;
      delta_t = 0.0001f;        // 1 [ms]
      samplingPeriod = 0.01f;  //   80 [ms]
      gravity = 500f;
      damp = true;
   }
    /**
    * Actualiza el delta de la simulacion.
    * @param delta Nuevo valor de delta
    */
   public void setDelta_t(float delta) {
      delta_t = delta;
   }
    /**
    * Actualiza el tiempo de muestreo de la simulacion.
    * @param sp Nuevo valor para el tiempo de muestreo
    */
   public void setSamplingPeriod (float sp) {
      samplingPeriod = sp;
      if (pase!=null)
         if (pase.isRunning()){
            stop();
            start();
         }   
   }
    /**
    * Actualiza la aceleracion de gravedad usada en la simulacion
    * @param g Nuevo valor la gravedad
    */
   public void setGravity(float g){
       gravity = g;
   }
    /**
    * Si la amortiguacion esta apagada es encendidia y viceversa.
    */
   public void setDamping(){
       damp = !damp;
   }
    /**
    * Inicia la simulacion
    */
   public void start() {
      t=0;
      pase = new Timer((int)(samplingPeriod*1000), this);
      pase.start();
   }
   /**
    * Detiene la simulacion, siempre que exista este corriendo.
    */
   public void stop(){
      if (pase!=null)
         pase.stop();
   }
   /**
    * Solicita la actualizacion de los eventos a la configuracion existente.
    * @param e Evento
    */
   public void actionPerformed (ActionEvent e) {
      float nextStop=t+samplingPeriod;
      for (; t<nextStop; t+=delta_t){
         bsConfig.computeNextState(delta_t,gravity);
         if (damp) bsConfig.checkNext(delta_t);
         bsConfig.updateState(damp);
      }
   }
}