/**
 * Clase para los vectores de dos dimensiones usados.
 */
public class Vector2D implements Cloneable {
   private double x, y;  // we will use cartesian coordinates
   /**
    * Construye un vector con ambas coordenadas incializadas en 0.
    */
   public Vector2D () {
      x = y = 0;
   }
   /**
    * Construye un vector con las coordenadas especificadas.
    * @param x Posicion x del vector.
    * @param y Posicion y del vector.
    */
   public Vector2D (double x, double y) {
      this.x = x;
      this.y = y;
   }
   /**
    * Obtiene el valor de x del vector
    * @return x.
    */
   public double getX(){
      return x;
   }
   /**
    * Obtiene el valor de y del vector
    * @return y.
    */
   public double getY(){
      return y;
   }
   /**
    * Actualiza los valores de x e y a los valores dados.
    * @param x Nuevo valor para x.
    * @param y Nuevo valor para y.
    */
   public void moveTo(double x, double y){ // to set a new position
      this.x=x;
      this.y=y;
   }
   /**
    * Suma vectorial
    * @param v Vector a sumar con el vector actual.
    * @return Nuevo vector con el resultado obtenido.
    */
   public Vector2D plus(Vector2D v) {
      try {
         if (v==null) return (Vector2D) clone();
      } catch (CloneNotSupportedException e){
         return null;
      }
      return new Vector2D(x+v.x, y+v.y);
   }
    /**
    * Multiplicacion escalar del vector
    * @param scalar Escalar a multplicar.
    * @return Nuevo vector con el resultado obtenido.
    */
   public Vector2D times(double scalar) {
      return new Vector2D(x*scalar, y*scalar);
   }
    /**
    * Resta vectorial
    * @param v Vector a restar con el vector actual.
    * @return Nuevo vector con el resultado obtenido.
    */
   public Vector2D minus(Vector2D v) {
      try {
         if (v==null) return (Vector2D) clone();
      } catch (CloneNotSupportedException e){
         return null;
      }
      return new Vector2D(x-v.x, y-v.y);
   }
    /**
    * Obtiene el modulo del vector
    * @return Modulo obtenido.
    */
   public double module() {
      return Math.sqrt(x*x+y*y);
   }
   /**
    * Obtiene el angulo del vector con respecto al origen
    * @return Angulo obtenido.
    */
   public double angle() {  // just in case we need it
      if (x==0){      // for x=0, we can't use arctang.
          if(y>0) return Math.PI/2;
          else return -Math.PI/2;
      }
      if (x > 0) return Math.atan(y/x);
      return Math.PI + Math.atan(y/x);
   }
   /**
    * Obtiene el vector unitario de el vector
    * @return Nuevo vector con los valores obtenidos.
    */
   public Vector2D unitary() {
      if (module() == 0) return new Vector2D();
      return times(1/module());
   }
   /**
    * Metodo para obtener la descripcion del vector.
    * @return descripcion.
    */
   public static String getDescription() {
      return "(x,y)";
   }
   /**
    * Metodo para obtener la informacion de la posicion del vector
    * @return String con la posicion.
    */
   public String toString() {
      return x+","+y;
   }
}
   